{$FORM TForm2, Main.sfm}                                      

uses
  Classes, SysUtils, Graphics, Controls, Forms, Dialogs, StdCtrls,
  outline, FileCtrl, ComCtrls, ExtCtrls,
  CAM, 
  CADDocument,  
  CADConstants,
  CADFunctions,
  CADTypes,                 
  CommonUtils;
        
                       
const GE_FLT_MAX = 3.4E+38;
const GE_DBL_MAX = 1.7E+308;

// angles in radians                                              
//            3,14159265358979323846 2643383279 5028841971 6939937510 5820974944 5923078164 0628620899 8628034825 3421170679
const GE_PI = 3.14159265358979323846;    // 180
const GE_PI2 =(GE_PI / 2.0);    // 1,57079632679489661923    90
const GE_PI4 =(GE_PI / 4.0);    // 0,785398163397448309615   45
const GE_2PI =(GE_PI+GE_PI);    // 6,28318530717958647692   360
                          
// aka degrees
const GE_DEG1 =   GE_PI / 180.0;
const GE_DEG2 =   GE_PI / 90.0;
const GE_DEG3 =   GE_PI / 60.0;
const GE_DEG4 =   GE_PI / 45.0;
const GE_DEG5 =   GE_PI / 36.0;
const GE_DEG10 =  GE_PI / 18.0;             
const GE_DEG20 =  GE_PI /  9.0;
const GE_DEG22_5 = GE_PI /  8.0;
const GE_DEG30 =  GE_PI /  6.0;
const GE_DEG36 =  GE_PI /  5.0;
const GE_DEG45 =  GE_PI4;
const GE_DEG90 =  GE_PI2;                            
const GE_DEG135 = (GE_PI2+GE_PI4);
const GE_DEG180 = GE_PI;
const GE_DEG225 = (GE_PI+GE_PI4);
const GE_DEG270 = (GE_PI+GE_PI2);
const GE_DEG315 = (GE_2PI-GE_PI4);
const GE_DEG360 = GE_2PI;


// Radian -> Degree
const GE_RAD_TO_DEG  =  57.2957795130823208768;
// Degree -> Radian                        
const GE_DEG_TO_RAD  =  0.01745329251994329577;    

const  fESSICmdKerfInsRight       = '30';
const  fESSICmdKerfInsLeft        = '29';
const  fESSICmdKerfOutsRight      = '29';    
const  fESSICmdKerfOutsLeft       = '30';                                     
const  fESSICmdKerfOFF            = '38';
    
const  fESSICmdRapidMoveON        = '5';
const  fESSICmdRapidMoveOFF       = '6';
const  fESSICmdToolDown           = '7';
const  fESSICmdToolUp             = '8';
const  fESSICmdCommentON          = '3';
const  fESSICmdCommentOFF         = '4';
const  fESSICmdProgramStart       = '%';
const  fESSICmdProgramEnd         = '0';
const  fESSICmdAsoulutON          = '90';
const  fESSICmdRelativeON         = '91';


        
    
var    fLastKerfInfo: integer;
var    fTraceInfoList: TList;       
var    fCurrLineNum: integer;             
var    hLcDrv: integer;
var    hLcWnd: integer;
var    hBlock: integer;
var    fProgressBar: TProgressBar;
  
var    fInputFileName: string; 
var    InputList: TStringList;   

var    LayerNr    : integer; 
var    PenWidth   : integer;   
var    PenStyle   : integer;                
var    PenColor   : Integer;
var    BrushStyle : integer;
var    BrushColor : integer;
var    RowStyle   : integer;
var    Selected   : boolean;
var    ArcStyle   : integer;      
    
var    fUnitFactor: single;
var    CommentON  : boolean;
var    fRelativeProgram: boolean;
var    RapidMoveON: boolean;
var    CuttingON:   boolean;
var    fImportMoves: boolean;
var    fShowRapidMoves: boolean;
var    fShowDirection: boolean;
var    lastposx : TFloatType;       
var    lastposy: TFloatType;            



             
function AngleFrom2Points(cx, cy, x2, y2: TFloatType): TFloatType;
var a, b: TFloatType;
begin
  a := x2 - cx;
  b := y2 - cy;

  //a := RoundTo(a, -3);
  //b := RoundTo(b, -3);

  if a = 0 then a := 0.001;
  if b = 0 then b := 0.001;

  if a = 0 then
    if b = 0 then
      Result := 0
    else
      if b > 0 then //Result := IfThen(y > 0, PI/2, GE_DEG270)
        Result := GE_PI/2
      else
        Result := GE_DEG270 // DEG270 = PI*1.5 ???
  else                                                                
    if b = 0 then
      if a > 0 then //Result := IfThen(x > 0, 0, PI)
        Result := 0
      else
        Result := GE_PI                           
    else
      if a > 0 then
        if b > 0 then
          Result := arctan(b / a)
        else
          Result := arctan(b / a) + GE_PI*2
      else
        Result := arctan(b / a) + GE_PI;
end;  

function isLine(AStr: string) : boolean;
var idx1, idx2, SignCount, AStrLen: integer; val1, val2 : string;
begin
  result := false;
  AStrLen     := StrLen(AStr);
  SignCount   := cuCharCount(AStr, '+') + cuCharCount(AStr, '-');
  if SignCount <> 2 then exit;
  idx1        := 1;
  idx2        := cuSignIdx(cuSubString(AStr, 2, AStrLen)) + idx1;
  val1        := cuSubString(AStr, idx1, (idx2 - 1));
  val2        := cuSubString(AStr, idx2, AStrLen);
  if ((val1 = '+') or (val1 = '-')) then val1 := val1 + '0';
  if ((val2 = '+') or (val2 = '-')) then val2 := val2 + '0';
                                          
  result :=  (   
                 ( (AStr[1] = '+') or (AStr[1] = '-' ))                              and
                 ((val1='+') or (val1='-') or (StrToIntDef(val1, -9999999) <> -9999999))  and
                 ((val2='+') or (val2='-') or (StrToIntDef(val2, -9999999) <> -9999999))
              );
end;


function isArc(AStr: string): boolean;
var idx1, idx2, idx3, idx4, idx5, SignCount, AStrLen: integer;
    val1, val2, val3, val4: string;
begin
  result      := false;
  AStrLen     := StrLen(AStr);
  SignCount   := cuCharCount(AStr, '+') + cuCharCount(AStr, '-');
  if SignCount <> 5 then exit;     
  idx1        := 1;
  idx2        := cuSignIdx(cuSubString(AStr, 2, AStrLen)) + idx1;
  idx3        := cuSignIdx(cuSubString(AStr, idx2 + 1, AStrLen)) + idx2;
  idx4        := cuSignIdx(cuSubString(AStr, idx3 + 1, AStrLen)) + idx3;
  idx5        := AStrLen;
  val1        := cuSubString(AStr, idx1, (idx2 - 1));
  val2        := cuSubString(AStr, idx2, (idx3 - 1));
  val3        := cuSubString(AStr, idx3, (idx4 - 1));
  val4        := cuSubString(AStr, idx4, (idx5 - 1));
  if ((val1 = '+') or (val1 = '-')) then val1 := val1 + '0';
  if ((val2 = '+') or (val2 = '-')) then val2 := val2 + '0';
  if ((val3 = '+') or (val3 = '-')) then val3 := val3 + '0';
  if ((val4 = '+') or (val4 = '-')) then val4 := val4 + '0';

  result :=  (   ((AStr[1] = '+') or (AStr[1] = '-' ))       and
                 ((val1='+') or (val1='-') or (StrToIntDef(val1, -9999999) <> -9999999))  and
                 ((val2='+') or (val2='-') or (StrToIntDef(val2, -9999999) <> -9999999))  and
                 ((val3='+') or (val3='-') or (StrToIntDef(val3, -9999999) <> -9999999))  and
                 ((val4='+') or (val4='-') or (StrToIntDef(val4, -9999999) <> -9999999))
              );
end;
               

procedure DrawArc(ACX, ACY, Aradius, AStartAngle, AArcAngle: single; ADirection: string);
var hArc: integer;
begin
  hArc := CAD_BlockAddArc (hBlock, ACX, ACY, ARadius, AStartAngle, AArcAngle);

  if hArc > 0  then
  begin                                    
    //AddTraceInfo(hArc);
  end;
                                                       
  {if (ADirection = '-') then
    Arc2D.Direction := adCLOCKWISE
  else
    Arc2D.Direction := adCOUNTERCLOCKWISE;
  fCADCmp2D.AddObject(-1, Arc2D);
  }

end;               
                                                     
procedure FillArc(AStr: string);
var idx1, idx2, idx3, idx4, idx5, SignAnzahl, AStrLen: integer;
    val1, val2, val3, val4, hDirection: string;
    AAX, AAY, BBX, BBY, CCX, CCY, DDX, DDY: Single;
    a1, a2, arcangle, x1, y1, x2, y2, cx, cy, radius: single;
    hArc: integer;
begin
  AStrLen     := StrLen(AStr);
  idx1        := 1;                               
  idx2        := cuSignIdx(cuSubString(AStr, idx1 + 1, AStrLen)) + idx1;
  idx3        := cuSignIdx(cuSubString(AStr, idx2 + 1, AStrLen)) + idx2;
  idx4        := cuSignIdx(cuSubString(AStr, idx3 + 1, AStrLen)) + idx3;
  idx5        := AStrLen;     
  hDirection  := AStr[idx5];
  val1        := cuSubString(AStr, idx1, (idx2 - 1));   //EndPunk_X
  val2        := cuSubString(AStr, idx2, (idx3 - 1));   //EndPunk_Y
  val3        := cuSubString(AStr, idx3, (idx4 - 1));   //MittelPunkt_X
  val4        := cuSubString(AStr, idx4, (idx5 - 1));   //MittelPunkt_Y
  if ((val1 = '+') or (val1 = '-')) then val1 := '0';
  if ((val2 = '+') or (val2 = '-')) then val2 := '0';
  if ((val3 = '+') or (val3 = '-')) then val3 := '0';
  if ((val4 = '+') or (val4 = '-')) then val4 := '0';

  x1           := LastPosX; //StartPunktX
  y1           := LastPosY; //StartPunktY  
  cx           := StrToFloat((val3) * fUnitFactor);  //MittelPunktX
  cy           := StrToFloat((val4) * fUnitFactor);  //MittelPunktY                          
  x2           := StrToFloat((val1) * fUnitFactor);  //EndPunktX
  y2           := StrToFloat((val2) * fUnitFactor);  //EndPunktY

  radius := sqrt(sqr(cx) + sqr(cy));
                                                
  CCX := LastPosX;  //bogen Startpunkt.x
  CCY := LastPosY;  //bogen Startpunkt.y
  DDX :=  x2;         //bogen EndPunkt.x
  DDY :=  y2;         //bogen EndPunkt.y

  if fRelativeProgram then
  begin
    cx  :=  cx  + LastPosX;
    cy  :=  cy  + LastPosY;
    AAX :=  AAX + LastPosX;
    AAY :=  AAY + LastPosY;
    BBX :=  BBX + LastPosX;
    BBY :=  BBY + LastPosY;
    //CCX :=  CCX + LastPosX;
    //CCY :=  CCY + LastPosY;
    DDX :=  DDX + LastPosX;
    DDY :=  DDY + LastPosY;
  end;

  //a1 := GetAngleFromPoints(cx, cy, C.X, C.Y);
  //a2 := GetAngleFromPoints(cx, cy, D.X, D.Y);


  a1 :=  AngleFrom2Points(cx, cy, CCX, CCY);
  a2 :=  AngleFrom2Points(cx, cy, DDX, DDY);


  arcangle := a2 - a1;
  if (arcangle < 0) then arcangle := arcangle + GE_2PI
  else
    if (arcangle > GE_2PI) then arcangle := arcangle - GE_2PI;

  if hDirection = '-' then
    arcangle := arcangle - 2*GE_PI;
                             

  if (CCX = DDX) and (CCY = DDY) then
  begin
    if hDirection = '-' then
      arcangle :=  -1 * 2*GE_PI
    else
      arcangle := 2*GE_PI;       
  end;

  if a1 = a2 then
     arcangle := 2*GE_PI;
  if (not RapidMoveON) then
  begin
    //DrawArc(cx, cy, radius, a1, arcangle, hDirection);
    hArc := CAD_BlockAddArc (hBlock, cx, cy, radius, a1, arcangle);

    if  arcangle > 0 then
      CAD_PropPutFloat(hArc, CAD_PROP_ARC_CCW, 1)
    else
      CAD_PropPutFloat(hArc, CAD_PROP_ARC_CCW, 0);    

  end;
  LastPosX := DDX;            
  LastPosY := DDY;
end;

procedure DrawLine(AX1, AY1, AX2, AY2: single);
var hLine: integer;
begin
  //if CuttingON then
  //begin
    hLine := CAD_BlockAddLine (hBlock, AX1, AY1, AX2, AY2);
    //AddTraceInfo(hLine);
  //end;
end;
    
procedure FillLine(AStr: string);
var idx1, idx2, SignAnzahl, AStrLen: integer; val1, val2 : string;
    x1, y1, x2, y2: single;
begin
  AStrLen     := StrLen(AStr);
  idx1        := 1;
  idx2        := cuSignIdx(cuSubString(AStr, idx1 + 1, AStrLen)) + idx1;
  val1        := cuSubString(AStr, idx1, (idx2 - 1));
  val2        := cuSubString(AStr, idx2, AStrLen);
  if ((val1 = '+') or (val1 = '-')) then val1 := '0';
  if ((val2 = '+') or (val2 = '-')) then val2 := '0';

  x1           := LastPosX;
  y1           := LastPosY;
  x2           := StrToFloat(val1) * fUnitFactor;
  y2           := StrToFloat(val2) * fUnitFactor;

  if fRelativeProgram then
  begin
    x2 := x2 + x1;         
    y2 := y2+ y1;
  end;

  if (not RapidMoveON) then
    DrawLine(x1, y1, x2, y2);
  LastPosX := x2;
  LastPosY := y2;
end;

          
procedure  ProcessEntitys;
var i, hHF: integer; hStr: string;
begin
  //lastposx := 0;
  //lastposy := 0;
  if fProgressBar <> nil then
  begin                                        
    fProgressBar.Position := 0;
    fProgressBar.Min      := 0;
    fProgressBar.Max      := InputList.Count -1;
    fProgressBar.Step     := 1;
  end;                                     

  for i:= 0 to (InputList.Count -1) do
  begin
    fCurrLineNum := i + 1;
    hStr := InputList[i];
    if (hStr <> '') then
    begin                                  
      if (hStr = fESSICmdKerfOFF) then
        fLastKerfInfo := ord(ktNone)
      else if (hStr = fESSICmdKerfInsRight) then
        fLastKerfInfo := ord(ktInsRight)
      else if (hStr = fESSICmdKerfInsLeft) then
        fLastKerfInfo := ord(ktInsLeft)
      else if (hStr = fESSICmdKerfOutsRight) then
        fLastKerfInfo := ord(ktOutsRight)
      else if (hStr = fESSICmdKerfOutsLeft) then
        fLastKerfInfo := ord(ktOutsLeft)
      else if (hStr = fESSICmdRapidMoveON) then
        RapidMoveON := true
      else if (hStr = fESSICmdRapidMoveOFF) then
        RapidMoveON := false
      else if (hStr = fESSICmdToolDown) then
        CuttingOn := true
      else if (hStr = fESSICmdToolUp) then
        CuttingOn := false
      else if  (hStr = fESSICmdAsoulutON) then
        fRelativeProgram := false
      else if  (hStr = fESSICmdRelativeON) then
        fRelativeProgram := true
      else if isLine(hStr) then
        FillLine(hStr)
      else  if isArc(hStr) then
        FillArc(hStr)
      //else FillErrRec(hStr);   
     end;
     if fProgressBar <> nil then
        fProgressBar.StepIt; 
    hStr := '';
  end;
  if fProgressBar <> nil then
    fProgressBar.Position := 0;
end;
           
procedure init;
begin
  //CAD_BlockClear(hBlock, 0);   
  CAD_BlockSelectEnt (hBlock, 0, 1);
  CAD_BlockSelErase(hBlock);
  fProgressBar  := ProgressBar1;      
  InputList     := TStringList.Create;             
  fLastKerfInfo := 0;          
  //fTraceInfoList: TList;       
  fCurrLineNum := 0;             
  fInputFileName := '';       
  fUnitFactor := 0.1;               
  CommentON  := false;
  fRelativeProgram := true;       
  RapidMoveON := true                  
  CuttingON := false;                
  lastposx := 0;                                         
  lastposy := 0;                                          
end;   

procedure Import(AFileName: string; ABlock: integer);
begin
  if FileExists(AFileName) then
  begin
    hBlock := ABlock;
    init;  
    InputList.LoadFromFile(FileListBox1.FileName);                         
    ProcessEntitys;   
    CAD_BlockUpdate (CADDocument.hBlock, 1, 0);
    CAD_WndExeCommand(CADDocument.hLcWnd, CAD_CMD_ZOOM_EXT, 0 );      
  end;                              
  InputList.Free;                                
end;
                                        
                                                                                                       
procedure FileListBox1Click(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);          
begin                                   
  import(FileListBox1.FileName, CADDocument.hBlock);
end;
                                   
                
