unit mDINExport;

interface

uses Classes, SysUtils, Dialogs, CadIntrf2D, tools;

var
  Output: TStringList;
  DestFileName,
  tmpOutputFileName: string;
  LastKerfInfo: string;
  KerfON,
  ProgramKerf: boolean;
  UnitFactor: double;
  Kerf : double;
  NewLine: string;
  CurrLineCount: integer;
  //DIN
  BlockFigures,
  LineNumON: boolean;
  LineNumStart,
  LineNumStep : integer;
  LineNumPrefix: string;
  ReturnReferencePoint,
  RelativeCoords,
  Comment: boolean;
  
  LastElementID: integer;

  const MIN_FIGURE_DISTANCE = 0.09;

implementation

procedure MainLoop;
var i, FiguresCount: integer; hKerfInfo: string;
begin
  LastKerfInfo  := ''; //CAM_DEFAULT_KERF_STR;
  LastElementID := -1; // Ungltig
  ProcessProgramStart;
  FiguresCount := GetFiguresCount;
  ProgressBarReset(0, FiguresCount, 1);
  for i := 0 to FiguresCount - 1  do
  begin
    if IsCounturStart(i) then
    begin
      CreateRapidMove(i);
      LastKerfInfo := ''; //CAM_DEFAULT_KERF_STR;
      hKerfInfo := GetFigureKerfInfo(i);
      ProcessCounturStart(hKerfInfo, LastKerfInfo);
      LastKerfInfo :=  hKerfInfo;
    end;
    ProcessFigure(i);
    if IsCounturEnd(i) then
      ProcessCounturEnd;
    ProgressBarStep;
  end;
  ProgressBarReset(0, FiguresCount, 1);
  ProcessProgramEnd;
end;

procedure CreateRapidMove(AIndex: integer);
var spx, spy, epx, epy: double; hDirection: word; hNextDirection, hSide:word;
begin
  epx := GetFigureSP_X(AIndex);
  epy := GetFigureSP_Y(AIndex);
  if (AIndex = 0) then
  begin
     spx := 0;
     spy := 0;
  end else
  begin
    spx := GetFigureEP_X(AIndex - 1);
    spy := GetFigureEP_Y(AIndex - 1);
  end;
  if   ((abs(spx - epx) > MIN_FIGURE_DISTANCE) or (abs(spy - epy) > MIN_FIGURE_DISTANCE)) then
    OutputRapidMoveData(spx, spy, epx, epy);
end;

function ProcessCounturStart(AKerfInfo, ALastKerfInfo: string): string;
begin
  OutputhRapidMoveOFFData;
  ProcessKerf(AKerfInfo, ALastKerfInfo);
  OutputToolDownData;
end;

procedure OutputhRapidMoveOFFData;
var hStr: string;
begin
  hStr :=  RapidMoveOFF;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

procedure OutputToolDownData;
var hStr: string;
begin
  hStr := ToolDown;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

procedure OutputRapidMoveData(AStartPointX, AStartPointY, AEndPointX, AEndPointY: double);
var hEndPointX, hEndPointY: double;  hStr: string; hLineNum: string;
begin
  hStr := ProcessLine(AStartPointX, AStartPointY, AEndPointX, AEndPointY);
  hStr :=  RapidMoveON + ' ' + hStr;
  if (hStr <> '') then
  begin
    output.Add(FormatOutput(hStr));
    LastElementID := 1;
  end;
end;

procedure ProcessFigure(AIndex: integer);
var hKerfInfo, hClassName: string;  hArcDirection: word;
spx, spy, epx, epy, cpx, cpy: double;  hLineData, hArcData: string;
begin
  spx  := GetFigureSP_X[Aindex];
  spy  := GetFigureSP_Y[Aindex];
  epx  := GetFigureEP_X[Aindex];
  epy  := GetFigureEP_Y[Aindex];

  hClassName  :=  GetFigureClassName[AIndex];
  hKerfInfo   :=  GetFigureKerfInfo[AIndex];
  if (hClassName = 'TLine') then
  begin
    ProcessKerf(hKerfInfo, LastKerfInfo);
    hLineData := ProcessLine(spx, spy, epx, epy);
    OutputLineData(hLineData);
    LastKerfInfo :=  hKerfInfo;
  end;
  if (hClassName = 'TArc') then
  begin
    ProcessKerf(hKerfInfo, LastKerfInfo);
    LastKerfInfo :=  hKerfInfo;
    cpx  := GetFigureCP_X[Aindex];
    cpy  := GetFigureCP_Y[Aindex];
    hArcDirection := GetFigureDirection[Aindex];
    hArcData := ProcessArc(cpx, cpy, spx, spy, epx, epy, hArcDirection);
    case  hArcDirection of
      LEFTDIRECTION:  OutputCWArcData(hArcData);
      RIGHTDIRECTION: OutputCCWArcData(hArcData);
    end;
  end;
  if (hClassName = 'TCircle') then
  begin
    {ProcessKerf(hKerfInfo, LastKerfInfo);
    LastKerfInfo :=  hKerfInfo;
    cpx  := CAD.FigureCP_X[Aindex];
    cpy  := CAD.FigureCP_Y[Aindex];
    ProcessKerf(hKerfInfo, LastKerfInfo);
    //ProcessCircle(cpx, cpy, spx, spy, epx, epy);}
  end;
end;

procedure OutputLineData(ALineData: string);
var hStr: string;
begin
  hStr := ALineData;
  if (not BlockFigures) then
    hStr :=  LineON + '  ' + hStr
  else
    if (LastElementID <> 2) then
       hStr :=  LineON + ' ' + hStr
    else hStr := '    ' + hStr;  //Block wird eigerckt.
  if (hStr <> '') then
  begin
    output.Add(FormatOutput(hStr));
    LastElementID := 2;
  end;
end;

procedure OutputCCWArcData(AArcData: string);
var hStr: string;
begin
  hStr := AArcData;
  if (not BlockFigures) then
    hStr := CCWArcON  + ' ' + hStr
  else
    if (LastElementID <> 3) then
       hStr := CCWArcON  + ' ' + hStr
    else hStr := '    ' + hStr;  //Block wird eigerckt.
  if (hStr <> '') then
  begin
    output.Add(FormatOutput(hStr));
    LastElementID := 3;
  end;
end;

procedure OutputCWArcData(AArcData: string);
var hStr: string;
begin
  hStr := AArcData;
  if (not BlockFigures) then
    hStr := CWArcON  + '  ' + hStr
  else
    if (LastElementID <> 4) then
         hStr := CWArcON  + ' ' + hStr
    else hStr := '    ' + hStr;  //Block wird eigerckt.
  if (hStr <> '') then
  begin
    output.Add(FormatOutput(hStr));
    LastElementID := 4;
  end;
end;

procedure ProcessCircle;
begin

end;

function IsCounturStart(AFigureIndex: integer): boolean;
var hRes: boolean;
begin
  hRes := false;
  if (AFigureIndex = 0) then
     hRes := true
  else
    if (not GetCombinedFigures[AFigureIndex, AFigureIndex-1]) then
      hRes := true;
  result := hRes;
end;

function IsCounturEnd(AFigureIndex: integer): boolean;
var hRes: boolean; FiguresCount: integer;
begin
  hRes := false;
  FiguresCount := GetFiguresCount;
  if (AFigureIndex = FiguresCount - 1) then
    hRes := true
  else
    if (not GetCombinedFigures[AFigureIndex+1, AFigureIndex]) then
      hRes := true;
  result := hRes;
end;

function GetDecimalSeparator: string;
begin
  result := '.';
end;

function GetNewLineNum: string;
var hStr: string;
begin
  if (CurrLineCount = 0) then
    hStr := LineNumPrefix + IntToStr(LineNumStart)
  else
    hStr := LineNumPrefix +  IntToStr(((CurrLineCount) *  LineNumStep) + LineNumStart);
  result := hStr;
end;

function CommentON: string;
begin
  result := '3';
end;

function CommentOFF: string;
begin
  result := '4';
end;

function CommentText: string;
var hStr: string;
begin
  {CurrLineCount := CurrLineCount + 1;
  hStr   := GetNewLineNum        + ' ' + CommentON + 'PostProcessor-Name:     '  + PPDesc   + CommentOFF + NewLine;
  CurrLineCount := CurrLineCount + 1;
  hStr   := hStr + GetNewLineNum + ' ' + CommentON + 'PostProcessor-FileName: '  + PPFileName + CommentOFF + NewLine;
  CurrLineCount := CurrLineCount + 1;
  hStr   := hStr + GetNewLineNum + ' ' + CommentON + 'Source-FileName:        '  + SourceFileName + CommentOFF + NewLine;
  CurrLineCount := CurrLineCount + 1;
  hStr   := hStr + GetNewLineNum + ' ' + CommentON + 'Output-FileName:        '  + DestFileName+ CommentOFF + NewLine;
  CurrLineCount := CurrLineCount + 1;
  hStr   := hStr + GetNewLineNum + ' ' + CommentON + 'PostProcessor-Author:   '  + PPAuthor + CommentOFF + NewLine;
  CurrLineCount := CurrLineCount + 1;
  hStr   := hStr + GetNewLineNum + ' ' + CommentON + 'Date:                   '  + DateTimeToStr(now)  + CommentOFF;
  result := hStr; }
end;


procedure ProcessProgramStart;
var hStr: string;
begin
  if (Comment = true) then
  begin
    hStr  := CommentText;
    output.Add(hStr);
  end;
  OutputRetToRefPointData;
  OutputRelAbsCoordinates;
end;

procedure OutputRetToRefPointData;
var hStr: string;
begin
  if ReturnReferencePoint then
  begin
    hStr := ReferencePointReturn;
    if (hStr <> '') then
      output.Add(FormatOutput(hStr));
  end;
end;

procedure OutputRelAbsCoordinates;
var hStr: string;
begin
  if RelativeCoords then
  begin
    hStr := IncrementalCoordinates;
    if (hStr <> '') then
      output.Add(FormatOutput(hStr));
  end
  else begin
    hStr := AbsoluteCoordinates;
    if (hStr <> '') then
      output.Add(FormatOutput(hStr));
  end;
end;

function RapidMoveON: string;
begin
  result := 'G00';
end;

function LineON: string;
begin
  result := 'G01';
end;

function CWArcON: string;
begin
  result := 'G02';
end;

function CCWArcON: string;
begin
  result := 'G03';
end;

function RapidMoveOFF: string;
begin
  result := '';
end;

function ToolDown: string;
begin
  result := 'M10';
end;

function ToolUp: string;
begin
  result := 'M11';
end;

function KerfOFF: string;
begin
  result := 'M40';
end;

function InsRightKerf: string;
begin
  result:= 'G42';
end;

function InsLeftKerf: string;
begin
  result:= 'G41';
end;

function OutsRightKerf: string;
begin
  result:= 'G42';
end;

function OutsLeftKerf: string;
begin
  result:= 'G41';
end;

function AbsoluteCoordinates: string;
begin
  result := 'G90';
end;

function IncrementalCoordinates: string;
begin
  result := 'G91';
end;

function ReferencePointReturn: string;
begin
  result := 'G28';
end;

function ProgramEnd: string;
begin
  result := 'M30';
end;

procedure ProcessKerf(AKerfInfo, ALastKerfInfo: string);
var hStr: string;
begin
  hStr := '';
  if  (AKerfInfo <> ALastKerfInfo) then
  begin
    if KerfON then
    begin
      if (AKerfInfo = CAM_KERF_INS_RIGHT_STR)       then
        hStr := InsRightKerf
      else if (AKerfInfo = CAM_KERF_INS_LEFT_STR)   then
         hStr := InsLeftKerf
      else if (AKerfInfo = CAM_KERF_OUTS_RIGHT_STR) then
        hStr := OutsRightKerf
      else if (AKerfInfo = CAM_KERF_OUTS_LEFT_STR)  then
        hStr := OutsLeftKerf
      else
        hStr := OutsLeftKerf;
      if ProgramKerf then
        hStr := hStr + '+' + FloatToStr(UnitFactor * Kerf);
    end;
  end;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

procedure ProcessCounturEnd;
begin
  OutputToolUpData;
  OutPutKerfOFFData;
end;

procedure OutputToolUpData;
var hStr: string;
begin
  hStr := ToolUp;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

procedure OutPutKerfOFFData;
var hStr: string;
begin
  hStr := KerfOFF;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

function ProcessLine(AStartPointX, AStartPointY, AEndPointX, AEndPointY: double): string;
var hEndPointX, hEndPointY: double;  hStr: string; hLineNum: string;
begin
  hEndPointX :=  UnitFactor * AEndPointX;
  hEndPointY :=  UnitFactor * AEndPointY;
  if RelativeCoords  then
  begin
    hEndPointX := hEndPointX - AStartPointX;
    hEndPointY := hEndPointY - AStartPointY;
  end;
  hStr := 'X' +  FormatFloat('0.00', hEndPointX) + ' ' +
          'Y' +  FormatFloat('0.00', hEndPointY);
  result := hStr;
end;

function ProcessArc(ACenterPointX, ACenterPointY, AStartPointX, AStartPointY, AEndPointX, AEndPointY: double; ADirection: word): string;
var Ex, Ey, Cx, Cy: double;  ExStr, EyStr, CxStr, CyStr, hStr: string; hDirection: string;
begin
   //if (ADirection = LEFTDIRECTION)  then hDirection := CCWArcON;
   //if (ADirection = RIGHTDIRECTION) then hDirection := CWArcON;

   if (RelativeCoords) then
   begin
     Ex := UnitFactor * (AEndPointX - AStartPointX);
     Ey := UnitFactor * (AEndPointY - AStartPointY);
   end else
   begin
     Ex := UnitFactor * AEndPointX;
     Ey := UnitFactor * AEndPointY;
   end;
   //CenterPunk ist immer Relativ.  ??
   Cx := UnitFactor * (ACenterPointX - AStartPointX);
   Cy := UnitFactor * (ACenterPointY - AStartPointY);

  //if (Ex = 0) then ExStr := ''
  {else}             ExStr := 'X' +  FormatFloat('0.00', Ex);

  //Ey :=  Ey * -1;   //spiegeln
  //if (Ey = 0) then EyStr := ''
  {else}             EyStr := 'Y' +  FormatFloat('0.00', Ey);

  //if (Cx = 0) then CxStr := ''
  {else}             CxStr := 'I' +  FormatFloat('0.00', Cx);

  //Cy :=  Cy * -1;   //spiegeln
  //if (Cy = 0) then CyStr := ''
  {else}             CyStr := 'J' +  FormatFloat('0.00', Cy);

  hStr := '';
  if  (ExStr <> '') then  hStr := hStr + ExStr + ' ';
  if  (EyStr <> '') then  hStr := hStr + EyStr + ' ';
  if  (CxStr <> '') then  hStr := hStr + CxStr + ' ';
  if  (CyStr <> '') then  hStr := hStr + CyStr;
  result := hStr;
end;

procedure ProcessProgramEnd;
var hStr: string;
begin
  hStr := ProgramEnd;
  if (hStr <> '') then
    output.Add(FormatOutput(hStr));
end;

function FormatOutput(AStr: string): string;
var hStr: string;
begin
  hStr := AStr;
  if LineNumON then
  begin
    hStr := GetNewLineNum  + ' ' + hStr;
    CurrLineCount := CurrLineCount + 1;
  end;
  result := hStr;
end;

procedure init(AOtputFileName: string);
begin
    output         := TStringList.Create;
    Comment        := false;
    UnitFactor     := 1;
    KerfON         := true;
    ProgramKerf    := false;
    Kerf           := 1.2;
    NewLine        := chr(13) + chr(10);
    CurrLineCount  := 0;
  //DIN
    BlockFigures   := true;
    LineNumON      := true;
    LineNumStart   := 100;
    LineNumStep    := 1;
    LineNumPrefix  := 'N';
    ReturnReferencePoint :=  true;
    RelativeCoords := true;
    MainLoop;
    output.SaveToFile(AOtputFileName);
    output.free;
end;

var SaveDialog1: TSaveDialog;

begin
  //CreateConfigWindow;
  try
    SaveDialog1    := TSaveDialog.Create(nil);
    SaveDialog1.Title := 'Export As DIN GCode';
    if SaveDialog1.Execute then
    begin
      init(SaveDialog1.FileName);
      WriteLn('Fertig');
    end;
  finally
     SaveDialog1.Free;
  end;
end.


